<?php

namespace App\Http\Controllers;

use App\Models\Pacientes;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use GuzzleHttp\Client; // Importa la clase Client de Guzzle

class PacientesController extends Controller
{
    protected $firestoreUrl;

    public function __construct()
    {
        // Inicializa la URL de Firestore
        $this->firestoreUrl = 'https://firestore.googleapis.com/v1/projects/pixie-b4bc3/databases/(default)/documents';
    }
    // Mostrar todos los pacientes
    public function index(Request $request)
    {
        $pacientes = $this->fetchPacientesFromFirestore();

        // Filtrar pacientes según la búsqueda
        $pacientes = $this->filterPacientes($pacientes, $request->input('query'));

        // Enviar los pacientes a la vista
        return view('pacientes.index', compact('pacientes'));
    }

    // Método para obtener pacientes de Firestore
    private function fetchPacientesFromFirestore()
    {
        $firestoreUrl = 'https://firestore.googleapis.com/v1/projects/pixie-b4bc3/databases/(default)/documents/pacientes';
        $response = Http::get($firestoreUrl);

        // Comprobar si la solicitud falló
        if ($response->failed()) {
            return back()->withErrors(['error' => 'Error al obtener los pacientes desde Firestore']);
        }

        // Procesar la respuesta para extraer datos de pacientes
        $documents = $response->json()['documents'] ?? [];
        return $this->processPacienteDocuments($documents);
    }

    // Procesar documentos de pacientes
    private function processPacienteDocuments(array $documents)
    {
        $pacientes = [];
        foreach ($documents as $document) {
            $fields = $document['fields'] ?? [];
            $pacientes[] = [
                'id' => isset($document['name']) ? basename($document['name']) : 'N/A',
                'nombre' => $fields['name']['stringValue'] ?? 'N/A',
                'apellido' => $fields['lastName']['stringValue'] ?? 'N/A',
                'email' => $fields['email']['stringValue'] ?? 'N/A',
                'telefono' => $fields['phone']['stringValue'] ?? 'N/A',
                'ansiedad' => $fields['resultados']['mapValue']['fields']['ansiedad']['integerValue'] ?? 'N/A',
                'depresion' => $fields['resultados']['mapValue']['fields']['depresion']['integerValue'] ?? 'N/A',
                'estres' => $fields['resultados']['mapValue']['fields']['estres']['integerValue'] ?? 'N/A',
            ];
        }
        return $pacientes;
    }

    // Filtrar pacientes por nombre, apellido o email
    private function filterPacientes(array $pacientes, ?string $query)
    {
        if ($query) {
            return array_filter($pacientes, function ($paciente) use ($query) {
                return stripos($paciente['nombre'], $query) !== false ||
                    stripos($paciente['apellido'], $query) !== false ||
                    stripos($paciente['email'], $query) !== false;
            });
        }
        return $pacientes;
    }

    // Mostrar el formulario para crear un nuevo paciente
    public function create()
    {
        return view('pacientes.create');
    }

    public function store(Request $request)
    {
        // Validar los datos del formulario
        $request->validate([
            'nombre' => 'required|string|max:255',
            'apellido' => 'required|string|max:255',
            'email' => 'required|email|unique:pacientes,email',
            'password' => 'required|string|min:6', // Asegurarse de que haya un campo de confirmación
            'fecha_nacimiento' => 'required|date',
            'telefono' => 'nullable|string|max:15',
            'direccion' => 'nullable|string|max:255',
            'ansiedad' => 'nullable|integer',
            'depresion' => 'nullable|integer',
            'estres' => 'nullable|integer',
        ]);

        // Preparar los datos para Firestore
        $data = [
            'address' => ['stringValue' => $request->input('direccion')],
            'birthDate' => ['stringValue' => $request->input('fecha_nacimiento')],
            'email' => ['stringValue' => $request->input('email')],
            'lastName' => ['stringValue' => $request->input('apellido')],
            'name' => ['stringValue' => $request->input('nombre')],
            'password' => ['stringValue' => hash('sha256', $request->input('password'))], // Usar SHA-256 para la contraseña
            'phone' => ['stringValue' => $request->input('telefono')],
            'resultados' => [
                'mapValue' => [
                    'fields' => [
                        'ansiedad' => ['integerValue' => $request->input('ansiedad') ?? 0],
                        'depresion' => ['integerValue' => $request->input('depresion') ?? 0],
                        'estres' => ['integerValue' => $request->input('estres') ?? 0],
                    ]
                ]
            ]
        ];

        // URL de Firestore para agregar un nuevo documento en la colección 'pacientes'
        $firestoreUrl = $this->firestoreUrl . "/pacientes?documentId=" . $request->input('email');

        // Realiza la solicitud POST a Firestore
        $response = Http::post($firestoreUrl, [
            'fields' => $data
        ]);

        // Comprobar si la solicitud fue exitosa
        if ($response->failed()) {
            Log::error("Error al crear el paciente: ", ['response' => $response->body()]);
            return back()->withErrors(['error' => 'Error al crear el paciente.']);
        }

        // Guardar en la sesión el estado del usuario
        session([
            'SESSION_ACTIVE' => true,
            'USER_ID' => $request->input('email'), // Guardar el ID del usuario (aquí se puede usar email)
            'USER_NAME' => $request->input('nombre'),
        ]);

        return redirect()->route('pacientes.index')->with('success', 'Paciente creado exitosamente.');
    }




    public function show($id)
    {
        // Obtener el paciente desde Firestore
        $pacientes = $this->fetchPacientesFromFirestore();

        // Filtrar el paciente por ID
        $paciente = collect($pacientes)->firstWhere('id', $id);

        // Verificar si el paciente existe
        if (!$paciente) {
            return redirect()->route('pacientes.index')->withErrors(['error' => 'Paciente no encontrado.']);
        }

        // Crear un mensaje para la API de OpenAI usando datos del paciente
        $mensajeDiagnostico = "Genera recomendaciones para un paciente con nombre {$paciente['nombre']}, "
            . "apellido {$paciente['apellido']}, "
            . "con estrés {$paciente['estres']}, "
            . "ansiedad {$paciente['ansiedad']}, "
            . "y depresión {$paciente['depresion']} ten encuenta que los niveles son del 0 al 100 siendo 100 el maximo nivel de cada uno
        si es entre 60 para abajo no necesita mucha ayuda profesional, si sobre pasa de los 70 o 80 si quiero que recomiednes eso.";

        // Generar el diagnóstico usando la API de OpenAI
        $client = new Client(); // Aquí se crea una instancia del cliente de Guzzle
        $response = $client->post('https://api.openai.com/v1/chat/completions', [
            'headers' => [
                'Authorization' => 'Bearer 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',
                'Content-Type' => 'application/json',
            ],
            'json' => [
                'model' => 'gpt-3.5-turbo',
                'messages' => [
                    ['role' => 'user', 'content' => $mensajeDiagnostico],
                ],
                'max_tokens' => 4000,
                'temperature' => 0.7,
            ],
        ]);

        $result = json_decode($response->getBody(), true);
        $recomendaciones = $result['choices'][0]['message']['content'] ?? 'No se pudieron generar recomendaciones.';

        // Obtener diagnósticos relacionados al paciente (si es necesario)
        $diagnosticos = $this->fetchDiagnosticosFromFirestore($id);

        return view('pacientes.show', compact('paciente', 'diagnosticos', 'recomendaciones'));
    }



    // Método para obtener un paciente específico de Firestore
    private function fetchPacienteFromFirestoreById($id)
    {
        $firestoreUrl = "https://firestore.googleapis.com/v1/projects/pixie-b4bc3/databases/(default)/documents/pacientes/{$id}";
        $response = Http::get($firestoreUrl);

        // Comprobar si la solicitud falló
        if ($response->failed()) {
            return null; // Manejar el error como prefieras
        }

        $document = $response->json();
        return $this->processSinglePacienteDocument($document);
    }

    // Procesar un único documento de paciente
    private function processSinglePacienteDocument(array $document)
    {
        $fields = $document['fields'] ?? [];
        return [
            'id' => isset($document['name']) ? basename($document['name']) : 'N/A',
            'nombre' => $fields['name']['stringValue'] ?? 'N/A',
            'apellido' => $fields['lastName']['stringValue'] ?? 'N/A',
            'email' => $fields['email']['stringValue'] ?? 'N/A',
            'telefono' => $fields['phone']['stringValue'] ?? 'N/A',
            // Agrega otros campos que necesites
        ];
    }

    // Método para obtener diagnósticos de Firestore
    private function fetchDiagnosticosFromFirestore($pacienteId)
    {
        $firestoreUrl = "https://firestore.googleapis.com/v1/projects/pixie-b4bc3/databases/(default)/documents/diagnosticos?where=paciente_id==$pacienteId";
        $response = Http::get($firestoreUrl);

        // Comprobar si la solicitud falló
        if ($response->failed()) {
            return []; // Manejar el error como prefieras
        }

        // Procesar la respuesta para extraer diagnósticos
        $documents = $response->json()['documents'] ?? [];
        return $this->processDiagnosticoDocuments($documents);
    }

    // Procesar documentos de diagnósticos
    private function processDiagnosticoDocuments(array $documents)
    {
        $diagnosticos = [];
        foreach ($documents as $document) {
            $fields = $document['fields'] ?? [];
            $diagnosticos[] = [
                'id' => isset($document['name']) ? basename($document['name']) : 'N/A',
                'resultado' => $fields['resultado']['stringValue'] ?? 'N/A',
                'fecha' => $fields['fecha']['stringValue'] ?? 'N/A',
                // Agrega otros campos que necesites
            ];
        }
        return $diagnosticos;
    }




    // Mostrar el formulario para editar un paciente
    public function edit($id)
    {
        // Suponiendo que tienes un método para obtener el paciente de Firestore
        $pacienteData = $this->getPacienteFromFirestore($id);

        if (!$pacienteData) {
            return redirect()->route('pacientes.index')->withErrors(['error' => 'Paciente no encontrado.']);
        }

        return view('pacientes.edit', compact('pacienteData'));
    }


    public function getPacienteFromFirestore($email)
    {
        // Define la URL de Firestore para obtener el documento del paciente
        $firestoreUrl = $this->firestoreUrl . "/pacientes/$email";

        // Realiza la solicitud GET a Firestore
        $response = Http::get($firestoreUrl);

        // Verifica si la solicitud fue exitosa
        if ($response->failed()) {
            Log::error("Error al obtener el paciente: ", ['response' => $response->body()]);
            return null; // Retorna null si hay un error
        }

        // Obtiene los datos del paciente
        $data = $response->json()['fields'] ?? null;

        // Agregar el ID (email en este caso) a los datos
        if ($data) {
            $data['id'] = $email; // o usa cualquier otro identificador
        }

        return $data;
    }



    // Actualizar los datos de un paciente
    public function update(Request $request, $email)
    {
        // Validar los datos del formulario
        $request->validate([
            'nombre' => 'required|string|max:255',
            'apellido' => 'required|string|max:255',
            'email' => 'required|email|unique:pacientes,email,' . $email,
            'fecha_nacimiento' => 'required|date',
            'telefono' => 'nullable|string|max:15',
            'direccion' => 'nullable|string|max:255',
            'ansiedad' => 'nullable|integer',
            'depresion' => 'nullable|integer',
            'estres' => 'nullable|integer',
        ]);

        // Preparar los datos para Firestore
        $data = [
            'name' => ['stringValue' => $request->input('nombre')],
            'lastName' => ['stringValue' => $request->input('apellido')],
            'email' => ['stringValue' => $request->input('email')],
            'birthDate' => ['stringValue' => $request->input('fecha_nacimiento')],
            'phone' => ['stringValue' => $request->input('telefono')],
        ];

        // Agregar solo los campos que tienen valor
        $resultados = [];
        if ($request->has('ansiedad')) {
            $resultados['ansiedad'] = ['integerValue' => $request->input('ansiedad')];
        }
        if ($request->has('depresion')) {
            $resultados['depresion'] = ['integerValue' => $request->input('depresion')];
        }
        if ($request->has('estres')) {
            $resultados['estres'] = ['integerValue' => $request->input('estres')];
        }
        if (!empty($resultados)) {
            $data['resultados'] = ['mapValue' => ['fields' => $resultados]];
        }

        // URL de Firestore para actualizar el documento del paciente
        $firestoreUrl = $this->firestoreUrl . "/pacientes/$email";

        // Realiza la solicitud PATCH a Firestore
        $response = Http::patch($firestoreUrl, [
            'fields' => $data
        ]);

        // Comprobar si la solicitud fue exitosa
        if ($response->failed()) {
            Log::error("Error al actualizar el paciente: ", ['response' => $response->body()]);
            return back()->withErrors(['error' => 'Error al actualizar el paciente.']);
        }

        return redirect()->route('pacientes.index')->with('success', 'Paciente actualizado exitosamente.');
    }



    // Eliminar un paciente
    public function destroy(Pacientes $paciente)
    {
        $paciente->update(['estado' => 0]);

        return redirect()->route('pacientes.index')
            ->with('success', 'Paciente desactivado correctamente.');
    }

    // Mostrar los resultados del paciente
    private function getResultadosFromFirestore($userName)
    {
        $firestoreUrl = $this->firestoreUrl . "/emociones:runQuery";

        // Ajusta el valor de búsqueda
        $userName = trim($userName); // Eliminar espacios en blanco al principio y al final

        // Define la consulta
        $query = [
            'structuredQuery' => [
                'select' => [
                    ['fieldPath' => 'userName'],
                    ['fieldPath' => 'answers'],
                    ['fieldPath' => 'timestamp'],
                    ['fieldPath' => 'date'],
                    ['fieldPath' => 'stateDescription']
                ],
                'from' => [['collectionId' => 'emociones']],
                'where' => [
                    'fieldFilter' => [
                        'field' => ['fieldPath' => 'userName'],
                        'op' => 'EQUAL',
                        'value' => ['stringValue' => $userName]
                    ]
                ]
            ]
        ];

        $response = Http::post($firestoreUrl, $query);

        // Log de respuesta
        Log::info("Respuesta de Firestore: ", $response->json());

        if ($response->failed()) {
            Log::error("Error en la consulta a Firestore: ", [
                'status' => $response->status(),
                'body' => $response->body()
            ]);
            return null;
        }

        // Comprueba si hay documentos en la respuesta
        $documents = $response->json();

        // Verifica si hay documentos
        if (isset($documents) && count($documents) > 0) {
            return $documents[0]['document'];
        } else {
            Log::info("No se encontraron documentos para el usuario: $userName");
            return null;
        }
    }



    public function showResultados($userName)
    {
        // Suponiendo que tienes un método para obtener los resultados de Firestore
        $resultadosData = $this->getResultadosFromFirestore($userName);

        if (!$resultadosData) {
            return redirect()->route('pacientes.index')->withErrors(['error' => 'Resultados no encontrados.']);
        }

        // Convierte el campo 'answers' a un formato más usable
        $answers = $resultadosData['fields']['answers']['mapValue']['fields'];

        return view('pacientes.resultados', [
            'resultadosData' => [
                'date' => $resultadosData['fields']['date']['stringValue'],
                'stateDescription' => $resultadosData['fields']['stateDescription']['stringValue'],
                'timestamp' => $resultadosData['fields']['timestamp']['timestampValue'],
                'answers' => $answers, // Guarda las respuestas para la vista
            ]
        ]);
    }
}
